@@ -1,16 +1,17 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
  *)
 (**************************************************************)
 (**************************************************************)
-(* LOG.MLI *)
+(* DEBUG.MLI *)
 (* Author: Mark Hayden, 11/96 *)
 (**************************************************************)
+open Trans
 
-val client : Hsys.inet -> Hsys.port -> unit
-val server : Hsys.inet -> Hsys.port -> unit
+val client : Alarm.t -> inet -> port -> unit
+val server : Alarm.t -> port -> unit
