@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -13,5 +13,5 @@
 (**************************************************************)
 open Trans
 
-val init : View.full -> port -> bool -> (View.full * Appl_intf.t)
+val init : Alarm.t -> View.full -> port -> bool -> (View.full * Appl_intf.New.t)
 
