@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -11,16 +11,18 @@
 (* DES.MLI *)
 (* Authors: Mark Hayden, Ohad Rodeh, 3/97 *)
 (**************************************************************)
+open Trans
+(**************************************************************)
+(*
 type context
 type init
 type key = string
 
 type src = string
 type dst = string
-type ofs = int
-type len = int
 
 val key_len : int
 val init : key -> init -> bool -> context
 val update : context -> src -> ofs -> dst -> ofs -> len -> unit
 val final : context -> unit
+*)
