@@ -1,14 +1,14 @@
 /**************************************************************/
 /*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
  */
 /**************************************************************/
 #include <stdio.h>
-#include "mlvalues.h"
+#include "caml/mlvalues.h"
 #define PROTO_LIST(x) x
 #include "des.h"
 
@@ -25,7 +25,6 @@ value desml_CBCInit(
   DES_CBC_CTX *ctx ;
   unsigned char *key, *vi ;
   int encrypt ;
-  int ret ;
 
   ctx = (DES_CBC_CTX*) &Byte(ctx_v, 0) ;
   key = &Byte(key_v, 0) ;
