@@ -1,15 +1,16 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
  *)
 (**************************************************************)
 open Ensemble
-open Rules
+open Trans
 open Appl_intf
+open Rules
 
 type cellCoordinate = int*int (* the co-oridinates of a life cell *)
 
