@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -18,6 +18,6 @@ open Msgs
 (* creates a slave which connects to master "masterendpt" and   *)
 (* with with timer "time"                                     *)
 
-val initSlave: Endpt.id -> Time.t -> Appl_intf.t
+val initSlave: Endpt.id -> Time.t -> Appl_intf.Old.t
 
 
