@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -16,10 +16,10 @@ open Trans
 open Hsys
 open Util
 open View
-open Appl_intf
+open Appl_intf open Old
 (**************************************************************)
-let name = Trace.source_file "MTALK"
-let failwith s = failwith (Util.failmsg name s)
+let name = Trace.file "MTALK"
+let failwith s = Trace.make_failwith name s
 (**************************************************************)
 
 (* This function returns the interface record that defines
@@ -35,8 +35,9 @@ let intf (ls,vs) my_name alarm =
    * to sending it.
    *)
   let buffer 	= ref "" in
+  let leave     = ref false in
 
-  (* This is an association list of (endpt,string name) for every
+  (* This is an array of string names for every
    * member in the group.
    *)
   let names 	= ref [|my_name|] in
@@ -61,26 +62,44 @@ let intf (ls,vs) my_name alarm =
       printf "MTALK:about to read\n"
     ) ;
     let len = read stdin buf 0 (String.length buf) in
-    if len = 0 then 
-      exit 0 ;				(* BUG: should leave group *)
-    buffer := !buffer ^ (String.sub buf 0 len) ;
-    async () ;
-    if !verbose then (
-      printf "MTALK:got line\n"
+    if len = 0 then (
+      (* Mark leave, stopped reading from stdin, and
+       * request to send an asynchronous event.
+       *)
+      leave := true ;
+      Alarm.rmv_sock_recv alarm stdin ;
+      async () ;
+    ) else (
+      (* Append data to buffer and request to send an
+       * asynchronous event.
+       *)
+      buffer := !buffer ^ (String.sub buf 0 len) ;
+      async () ;
+      if !verbose then (
+	printf "MTALK:got line\n"
+      )
     )
   in
-  Alarm.add_sock alarm stdin (Handler0 get_input) ;
+  Alarm.add_sock_recv alarm name stdin (Handler0 get_input) ;
 
   (* If there is buffered data, then return a Cast action to
    * send it.
    *)
   let check_buf () =
-    if !buffer <> "" then (
-      let send = !buffer in
-      buffer := "" ;
-      [Cast(send)]
+    let data = 
+      if !buffer <> "" then (
+	let send = !buffer in
+	buffer := "" ;
+	[|Cast(send)|]
+      ) else (
+	[||]
+      )
+    in
+    if !leave then (
+      leave := false ;
+      Array.append data [|Control(Leave)|]
     ) else (
-      []
+      data
     )
   in
 
@@ -107,7 +126,7 @@ let intf (ls,vs) my_name alarm =
     check_buf ()
 
   and recv_send _ _ = failwith "error"
-  and block () = []
+  and block () = [||]
   and heartbeat _ =
     check_buf ()
   and block_recv_cast origin msg =
@@ -128,11 +147,11 @@ let intf (ls,vs) my_name alarm =
   and exit () =
     printf "(mtalk:got exit)\n" ;
     exit 0
-  in full {
+  in {
     recv_cast           = recv_cast ;
     recv_send           = recv_send ;
     heartbeat           = heartbeat ;
-    heartbeat_rate      = Time.of_float 10.00 ;
+    heartbeat_rate      = Time.of_int 10 ;
     block               = block ;
     block_recv_cast     = block_recv_cast ;
     block_recv_send     = block_recv_send ;
@@ -158,7 +177,7 @@ let run () =
    * Get default transport and alarm info.
    *)
   let view_state = Appl.default_info "mtalk" in
-  let alarm = Alarm.get () in
+  let alarm = Elink.alarm_get_hack () in
 
   (*
    * Choose a string name for this member.  Usually
@@ -180,6 +199,10 @@ let run () =
    *)
   let interface = intf view_state name alarm in
 
+  (* Wrap it with marshalling code.
+   *)
+  let interface = (Elink.get_appl_old_full name) interface in
+
 (*let interface = Appl_intf.debug interface in*)
 
   (*
