@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -16,12 +16,12 @@ open Tk
 open Util
 open Appl
 open View
-open Appl_intf
+open Appl_intf open Old
 open Proto
 (**************************************************************)
-let name = Trace.source_file "WB"
-let failwith s = failwith (Util.failmsg name s)
-let log = Trace.log name ""
+let name = Trace.file "WB"
+let failwith s = Trace.make_failwith name s
+let log = Trace.log name
 (**************************************************************)
 
 type coord = int
@@ -254,11 +254,11 @@ let intf (ls_init,vs_init) my_name alarm top do_migrate =
   (* This is the buffer used to store typed input prior
    * to sending it.
    *)
-  let msg_buf = Queue.create () in
+  let msg_buf = Queuee.create () in
   let msg_action a = 
     async () ;
     Alarm.schedule (Alarm.alarm alarm (fun _ ->())) (Time.zero) ;
-    Queue.add a msg_buf 
+    Queuee.add a msg_buf 
   in
   let msg_cast m = msg_action (Cast(m)) in
 
@@ -266,9 +266,9 @@ let intf (ls_init,vs_init) my_name alarm top do_migrate =
    * actions to send them (after packing them).
    *)
   let check_buf () =
-    let send = list_of_queue msg_buf in
-    Queue.clear msg_buf ;
-    send
+    let send = Queuee.to_list msg_buf in
+    Queuee.clear msg_buf ;
+    Array.of_list send
   in
 
   (* Pointers is an array containing locations of all the
@@ -443,7 +443,7 @@ let intf (ls_init,vs_init) my_name alarm top do_migrate =
     | Some obj -> Canvas.delete wb_cv [obj]
     | None -> ()
     ) !pointers ;
-    pointers := array_create name nmembers None
+    pointers := Array.create nmembers None
   in
 
   (* PTR_SET: Update one of the other member's pointers.
@@ -520,7 +520,7 @@ let intf (ls_init,vs_init) my_name alarm top do_migrate =
   let protocol_switch props' =
     props := props' ;
     let proto_id = Property.choose !props in
-    msg_action (Protocol(proto_id)) ;	(* install the protocol locally *)
+    msg_action (Control(Protocol(proto_id))) ;	(* install the protocol locally *)
     msg_cast (ProtocolSwitch(!props)) ;	(* broadcast the switch *)
   in
 
@@ -530,11 +530,13 @@ let intf (ls_init,vs_init) my_name alarm top do_migrate =
 
   let migrate_press _ =
     let addr = do_migrate "gulag" in
-    msg_action (Migrate(addr))
+    msg_action (Control(Migrate(addr)))
   in
 
   let rekey_press _ =
-    msg_action Rekey
+()(* Not supported right now.
+    msg_action (Control Rekey)
+*)
   in
 
   (* Reactions for various button presses.
@@ -603,7 +605,7 @@ let intf (ls_init,vs_init) my_name alarm top do_migrate =
       | ProtocolSwitch props' ->
 	  props := props' ;
 	  let proto_id = Property.choose !props in
-	  msg_action (Protocol(proto_id)) (* install the protocol locally *)
+	  msg_action (Control (Protocol(proto_id))) (* install the protocol locally *)
     )
   in
 
@@ -613,7 +615,7 @@ let intf (ls_init,vs_init) my_name alarm top do_migrate =
   and recv_cast_iov _ _ = failwith "recv_cast_iov"
 
   and recv_send _ _ = failwith "error"
-  and block () = []
+  and block () = [||]
   and heartbeat _ =
     check_buf ()
 
@@ -636,7 +638,7 @@ let intf (ls_init,vs_init) my_name alarm top do_migrate =
     (props,names)
   and unblock_view (ls,vs) (props',names') =
     names       := Array.of_list names' ;
-    view 	:= vs.view ;
+    view 	:= Arrayf.to_array vs.view ;
     rank 	:= ls.rank ;
     if_some props' (fun props' ->	(*HACK*)
       props       := props'
@@ -678,7 +680,7 @@ let intf (ls_init,vs_init) my_name alarm top do_migrate =
 
     (* Update the pointers to new view size.
      *)
-    ptr_reset (Array.length vs.view) ;
+    ptr_reset (Arrayf.length vs.view) ;
 
     (* Print current protocol in use.
      *)
@@ -690,8 +692,8 @@ let intf (ls_init,vs_init) my_name alarm top do_migrate =
     (* Print available protocols in this view. 
      *)
     let modes = Addr.modes_of_view vs.address in
-    let modes = Array.map Addr.string_of_id modes in
-    let modes = String.concat ":" (Array.to_list modes) in
+    let modes = Arrayf.map Addr.string_of_id modes in
+    let modes = String.concat ":" (Arrayf.to_list modes) in
     Label.configure trans_lab [Text ("Transports: "^modes)] ;
 
     (* Print security key in use.
@@ -699,16 +701,16 @@ let intf (ls_init,vs_init) my_name alarm top do_migrate =
     let key = Security.string_of_key_short vs.key in
     Label.configure key_lab [Text ("Key: "^key)] ;
 
-    (check_buf ()) @ [XferDone]
+    Array.append (check_buf ()) [|Control XferDone|]
 
   and exit () =
     printf "exiting\n"
 
-  in full ((*Appl_intf.debug*) {
+  in {
     recv_cast           = recv_cast ;
     recv_send           = recv_send ;
     heartbeat           = heartbeat ;
-    heartbeat_rate      = (Time.of_float 10.0) ;
+    heartbeat_rate      = (Time.of_int 10) ;
     block               = block ;
     block_recv_cast     = block_recv_cast ;
     block_recv_send     = block_recv_send ;
@@ -716,7 +718,7 @@ let intf (ls_init,vs_init) my_name alarm top do_migrate =
     block_install_view  = block_install_view ;
     unblock_view        = unblock_view ;
     exit                = exit
-  })
+  }
 
 (**************************************************************)
 
@@ -731,7 +733,7 @@ let init top view_state username do_migrate =
 
   (* Initialize the application interface.
    *)
-  let alarm = Alarm.get () in
+  let alarm = Elink.alarm_get_hack () in
   let interface = intf view_state username alarm top do_migrate in
   
   let up ev =
@@ -748,12 +750,16 @@ let init top view_state username do_migrate =
     | _ -> ()
   in
 
-  (* Initialize the Horus protocol stack, using the
+  (* Initialize the protocol stack, using the
    * interface, protocol stack, and transports chosen above.
    *)
-  let dn = Stacke.config_full interface view_state up in
+  let mbuf = Alarm.mbuf alarm in
+  let interface = Elink.get_appl_old_full name interface in
+  let interface = Elink.get name Elink.appl_compat_compat interface in
+  let interface = Elink.get_appl_power_newf name mbuf interface in
+  let glue = Arge.get Arge.glue in
+  let state = Layer.new_state interface in
+  let dn = Stacke.config_full glue alarm Addr.default_ranking state view_state up in
   ()
 
-
-
 (**************************************************************)
