@@ -1,7 +1,7 @@
 /**************************************************************/
 /*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -19,6 +19,7 @@
 #include "hot_msg.h"
 #include "hot_thread.h"
 #include "hot_sys.h"
+#include "hot_mem.h"
  
 struct hot_msg_ctl;
 struct hot_msg_body;
@@ -31,8 +32,9 @@ struct hot_msg_body;
 struct hot_msg_handle {
     struct hot_msg_ctl *ctl;	/* Points to the control struct */
     unsigned offset;
+    hot_msg_readDirection_t read_dir; /* Direction of reading */
     int index;
-    int magic;  
+    unsigned int magic;  
 };
 
 /* We support up to 32 aliases to a message (in practice, 1-2 suffice).
@@ -56,7 +58,7 @@ struct hot_msg_ctl {
     int need_release;		/* set if on_release needs to be called */
     char *data;			/* points to the beginning of data buffer */
     unsigned alloc_size;	/* size of the allocated data buffer */
-    int magic;
+    unsigned int magic;		/* MUST BE THE LAST FIELD, FOR DATA ALIGNMENT */
 };
 
 /* Size of a message buffer.  Max supported message size is
@@ -87,11 +89,18 @@ static struct {
   struct hot_msg_body *free_list;
   unsigned nfree;			/* # free messages */
   hot_lock_t mutex; 
+  hot_mem_t memory;
 } global;
 
 /* Return the first power of two larger than the given number.
  */
-static inline int hot_power2(unsigned n) {
+/* ijh: HP cc does not have inline */
+#ifdef INLINE_PRAGMA
+#pragma inline hot_power2
+#endif
+MSG_INLINE
+static
+int hot_power2(unsigned n) {
     unsigned p2 = 1;
     while (p2 <= n) p2 <<= 1;
     return p2;
@@ -99,16 +108,24 @@ static inline int hot_power2(unsigned n) {
 
 /* Release the data buffer of a big message.
  */
+#ifdef INLINE_PRAGMA
+#pragma inline hot_msg_ReleaseBigMsg
+#endif
+MSG_INLINE
 static void hot_msg_ReleaseBigMsg(void *arg) {
     struct hot_msg_ctl *ctl = (struct hot_msg_ctl*) arg;
     if (ctl == NULL || ctl->data == NULL)
 	hot_sys_Panic("hot_msg_ReleaseBigMsg:  internal inconsistency");
-    free(ctl->data);
+    hot_mem_Free(ctl->data);
 }
 
 /* Fit a message buffer to the specified size.  Allocate more memory if needed.
  * The message is locked at this point.
  */
+#ifdef INLINE_PRAGMA
+#pragma inline hot_msg_FitToSize
+#endif
+MSG_INLINE
 static void hot_msg_FitToSize(struct hot_msg_ctl *ctl, unsigned new_size) {
     if (new_size <= ctl->alloc_size) 
 	return;
@@ -121,10 +138,14 @@ static void hot_msg_FitToSize(struct hot_msg_ctl *ctl, unsigned new_size) {
      */
     if (!ctl->need_release) {
 	char *old_data = ctl->data;      
-	ctl->data = malloc(ctl->alloc_size);
+	ctl->data = hot_mem_Alloc(global.memory, ctl->alloc_size);
 	if (ctl->data == NULL)
 	    hot_sys_Panic("hot_msg_Write: malloc returned NULL");
 	
+	/* The beginning of a message buffer must be 4-byte aligned.
+	 */
+	assert(((unsigned)ctl->data) % 4 == 0);
+	
 	ctl->need_release = 1;
 	ctl->on_release = hot_msg_ReleaseBigMsg;
 	ctl->env = (void*) ctl;
@@ -135,9 +156,13 @@ static void hot_msg_FitToSize(struct hot_msg_ctl *ctl, unsigned new_size) {
     }
     
     else {
-	ctl->data = realloc(ctl->data, ctl->alloc_size);
+	ctl->data = hot_mem_Realloc(ctl->data, ctl->alloc_size);
 	if (ctl->data == NULL)
 	    hot_sys_Panic("hot_msg_Write: realloc returned NULL");
+
+	/* The beginning of a message buffer must be 4-byte aligned.
+	 */
+	assert(((unsigned)ctl->data) % 4 == 0);
     }
 }
 
@@ -155,15 +180,17 @@ static void hot_msg_Init() {
   if ((err = hot_lck_Lock(global.mutex)) != HOT_OK)
      hot_sys_Panic(hot_err_ErrString(err));
 
+  global.memory = hot_mem_AddChannel("hot_msg");
+
   /* Preallocate some messages and put them on free list.
    */
-  global.free_list = (struct hot_msg_body*) malloc(HOT_MSG_PREALLOC * 
-		     sizeof(struct hot_msg_body));
+  global.free_list = (struct hot_msg_body*) hot_mem_Alloc(
+      global.memory, HOT_MSG_PREALLOC * sizeof(struct hot_msg_body));
   if (global.free_list == NULL)
     hot_sys_Panic("hot_msg_init:malloc");
 
   for (i = 0; i < HOT_MSG_PREALLOC; i++) {
-      int k;
+      unsigned int k;
       global.free_list[i].ctl.magic = 0;
       for (k = 0; k < HOT_MSG_MAX_ALIASES; k++) {
 	  global.free_list[i].ctl.handles[k].magic = 0;
@@ -182,12 +209,16 @@ static void hot_msg_Init() {
 
 /* Create a message.  Returns the new message.
  */
+#ifdef INLINE_PRAGMA
+#pragma inline hot_msg_Create
+#endif
+MSG_INLINE
 hot_msg_t hot_msg_Create(void) {
   struct hot_msg_body *body;
   struct hot_msg_ctl *ctl;
   struct hot_msg_handle *handle;
   hot_err_t err;
-  int k;
+  unsigned int k;
   int new_msg = 0;
 
   if (!global.initialized)
@@ -210,7 +241,8 @@ hot_msg_t hot_msg_Create(void) {
     printf("** WARNING: hot_msg_Create: No messages left on the free list.\n");
     printf("** This might be caused by a memory leak\n");
     
-    body = (struct hot_msg_body *) malloc(sizeof(struct hot_msg_body));
+    body = (struct hot_msg_body *) hot_mem_Alloc(
+        global.memory, sizeof(struct hot_msg_body));
     if (body == NULL)      
       hot_sys_Panic("hot_msg_Create: malloc");
     new_msg = 1;
@@ -236,8 +268,16 @@ hot_msg_t hot_msg_Create(void) {
   ctl->writable = 1;
   ctl->size = 0;
   ctl->need_release = 0;
-  ctl->env = ctl->on_release = NULL;
+  /* ijh: HP cc complains so let's split this up */
+  /*ctl->env = ctl->on_release = NULL;*/
+  ctl->env = NULL;
+  ctl->on_release = NULL;  
   ctl->data = &body->data[0];
+
+  /* The beginning of a message buffer is 4-byte aligned.
+   */
+  assert(((unsigned)ctl->data) % 4 == 0);
+
   ctl->alloc_size = HOT_MSG_MAX_PREALLOC_SIZE;
   ctl->magic = HOT_MSG_CTL_MAGIC;
 
@@ -248,6 +288,7 @@ hot_msg_t hot_msg_Create(void) {
   handle = &ctl->handles[0];
   handle->ctl = ctl;
   handle->offset = 0;
+  handle->read_dir = HOT_MSG_READ_LEFT;
   handle->index = 0;
   handle->magic = HOT_MSG_HANDLE_MAGIC;
 
@@ -259,6 +300,10 @@ hot_msg_t hot_msg_Create(void) {
 
 /* Release a message.
  */
+#ifdef INLINE_PRAGMA
+#pragma inline hot_msg_Release
+#endif
+MSG_INLINE
 hot_err_t hot_msg_Release(hot_msg_t *msgp) {
     struct hot_msg_handle *handle;
     struct hot_msg_ctl *ctl;
@@ -331,6 +376,10 @@ hot_err_t hot_msg_Release(hot_msg_t *msgp) {
 
 /* Return total size of the message.
  */
+#ifdef INLINE_PRAGMA
+#pragma inline hot_msg_Size
+#endif
+MSG_INLINE
 hot_err_t hot_msg_Size(hot_msg_t msg, /*OUT*/ unsigned *sizep) {
   struct hot_msg_handle *handle = (struct hot_msg_handle*) msg;
   struct hot_msg_ctl *ctl;
@@ -358,6 +407,85 @@ hot_err_t hot_msg_Size(hot_msg_t msg, /*OUT*/ unsigned *sizep) {
   return HOT_OK;
 }
 
+/* Set the read direction.  By default, messages are read from left
+ * to right (first written/last read).  This can changed to read
+ * from left to right (first written/first read).
+ */
+hot_err_t hot_msg_ReadDirection(hot_msg_t msg, hot_msg_readDirection_t dir) {
+  struct hot_msg_handle *handle = (struct hot_msg_handle*) msg;
+
+  /* Consistency checks.
+   */
+  if (handle == NULL || handle->magic != HOT_MSG_HANDLE_MAGIC)
+    return hot_err_Create(0, "bad handle");
+
+  if (handle->ctl->magic != HOT_MSG_CTL_MAGIC)
+    return hot_err_Create(0, "bad message");
+  
+  handle->read_dir = dir;
+  return HOT_OK;
+}
+
+/* Move message pointer to the right to the next alignment boundary.
+ * The beginning of message data buffers is always 4-byte aligned.
+ */ 
+hot_err_t hot_msg_Align(hot_msg_t msg, unsigned nbytes) {
+  struct hot_msg_handle *handle = (struct hot_msg_handle*) msg;
+  struct hot_msg_ctl *ctl;
+  hot_err_t result = HOT_OK;
+  hot_err_t err;
+  unsigned pad;
+
+  /* Consistency check.
+   */
+  if (handle == NULL || handle->magic != HOT_MSG_HANDLE_MAGIC)
+    return hot_err_Create(0, "bad handle");
+  
+  ctl = handle->ctl;
+  if (ctl->magic != HOT_MSG_CTL_MAGIC)
+    return hot_err_Create(0, "bad message");
+  
+  pad = nbytes - ((unsigned)handle->offset % nbytes);
+  if (pad == nbytes) pad = 0;
+
+  if ((err = hot_lck_Lock(ctl->mutex)) != HOT_OK)
+    hot_sys_Panic(hot_err_ErrString(err));
+
+  /* If message is not writable, alignment must not increase the offset
+   * beyond the current boundary.
+   */
+  if (!ctl->writable) {
+    if (handle->offset + pad > ctl->size) {
+      result = hot_err_Create(0, "message boundary violation");
+      goto return_point;
+    }
+
+    handle->offset += pad;
+  }
+  else {
+    /* Consistency check.
+     */
+    if (ctl->size != handle->offset) {
+      result = hot_err_Create(0, "internal integrity violation");
+      goto return_point;
+    }
+    
+    /* Fit message buffer to the new size.
+     */ 
+    hot_msg_FitToSize(ctl, handle->offset + pad);
+
+    ctl->size += pad;
+    handle->offset = ctl->size;
+  }
+
+return_point:
+  
+  if ((err = hot_lck_Unlock(ctl->mutex)) != HOT_OK)
+    hot_sys_Panic(hot_err_ErrString(err));
+  
+    return result;
+}
+
 /* Seek to the specified position (for reads only).
  * The message becomes read-only.  
  * 
@@ -365,6 +493,10 @@ hot_err_t hot_msg_Size(hot_msg_t msg, /*OUT*/ unsigned *sizep) {
  * Seeks with positive offsets are to the right, and seeks with negative
  * offsets are to the left.
  */
+#ifdef INLINE_PRAGMA
+#pragma inline hot_msg_Seek
+#endif
+MSG_INLINE
 hot_err_t hot_msg_Seek(hot_msg_t msg, hot_msg_seek_t whence, unsigned offset) {
   struct hot_msg_handle *handle = (struct hot_msg_handle*) msg;
   struct hot_msg_ctl *ctl;
@@ -413,6 +545,10 @@ hot_err_t hot_msg_Seek(hot_msg_t msg, hot_msg_seek_t whence, unsigned offset) {
 
 /* Get the current read offset in the message.
  */
+#ifdef INLINE_PRAGMA
+#pragma inline hot_msg_GetPos
+#endif
+MSG_INLINE
 hot_err_t hot_msg_GetPos(hot_msg_t msg, /*OUT*/ unsigned *posp) {
   struct hot_msg_handle *handle = (struct hot_msg_handle*) msg;
 
@@ -431,19 +567,43 @@ hot_err_t hot_msg_GetPos(hot_msg_t msg, /*OUT*/ unsigned *posp) {
 /* Set read-offset pointer to the specified position.
  * The message becomes read-only.
  */
+#ifdef INLINE_PRAGMA
+#pragma inline hot_msg_SetPos
+#endif
+MSG_INLINE
 hot_err_t hot_msg_SetPos(hot_msg_t msg, unsigned offset) {
   struct hot_msg_handle *handle = (struct hot_msg_handle*) msg;
+  struct hot_msg_ctl *ctl;
+  hot_err_t result = HOT_OK;
+  hot_err_t err;
 
   /* Consistency check.
    */
   if (handle == NULL || handle->magic != HOT_MSG_HANDLE_MAGIC)
       return hot_err_Create(0, "bad handle");
 
-  if (handle->ctl->magic != HOT_MSG_CTL_MAGIC)
+  ctl = handle->ctl;
+  if (ctl->magic != HOT_MSG_CTL_MAGIC)
       return hot_err_Create(0, "bad message");
   
+  if ((err = hot_lck_Lock(ctl->mutex)) != HOT_OK)
+    hot_sys_Panic(hot_err_ErrString(err));
+
+  ctl->writable = 0;
+
+  if (offset < 0 || offset > ctl->size) {
+    result = hot_err_Create(0, "bad offset");
+    goto return_point;
+  }
+
   handle->offset = offset;
-  return HOT_OK;
+
+return_point:
+
+  if ((err = hot_lck_Unlock(ctl->mutex)) != HOT_OK)
+    hot_sys_Panic(hot_err_ErrString(err));
+
+  return result;
 }
 
 /* Make an alias of the specified message.  Both the original message
@@ -451,6 +611,10 @@ hot_err_t hot_msg_SetPos(hot_msg_t msg, unsigned offset) {
  *
  * In case of success, the pointer to the alias is returned in aliasp.
  */
+#ifdef INLINE_PRAGMA
+#pragma inline hot_msg_Alias
+#endif
+MSG_INLINE
 hot_err_t hot_msg_Alias(hot_msg_t msg, /*OUT*/ hot_msg_t *aliasp) {
     struct hot_msg_handle *handle = (struct hot_msg_handle*) msg;
     struct hot_msg_ctl *ctl;
@@ -473,7 +637,7 @@ hot_err_t hot_msg_Alias(hot_msg_t msg, /*OUT*/ hot_msg_t *aliasp) {
     if (ctl->nused == HOT_MSG_MAX_ALIASES)
 	result = hot_err_Create(0, "too many aliases");
     else {
-	int i;
+	unsigned int i;
 	for (i = 0; i < HOT_MSG_MAX_ALIASES; i++) {
 	    /* Use the first available handle.
 	     */
@@ -483,6 +647,7 @@ hot_err_t hot_msg_Alias(hot_msg_t msg, /*OUT*/ hot_msg_t *aliasp) {
 		alias = &ctl->handles[i];
 		alias->offset = handle->offset;
 		alias->ctl = ctl;
+		alias->read_dir = handle->read_dir;
 		alias->index = i;
 		alias->magic = HOT_MSG_HANDLE_MAGIC;
 		*aliasp = (hot_msg_t) alias;
@@ -507,6 +672,10 @@ hot_err_t hot_msg_Alias(hot_msg_t msg, /*OUT*/ hot_msg_t *aliasp) {
  *
  * In case of success, the pointer to the copy is returned in copyp.
  */
+#ifdef INLINE_PRAGMA
+#pragma inline hot_msg_Copy
+#endif
+MSG_INLINE
 hot_err_t hot_msg_Copy(hot_msg_t msg, /*OUT*/ hot_msg_t *copyp) {
   struct hot_msg_handle *handle_src = (struct hot_msg_handle*) msg;
   struct hot_msg_ctl *ctl_src;
@@ -549,6 +718,10 @@ hot_err_t hot_msg_Copy(hot_msg_t msg, /*OUT*/ hot_msg_t *copyp) {
 /* Create a read-only message with the specified buffer as its contents.
  * When the message is to be released, the specified function is invoked.
  */
+#ifdef INLINE_PRAGMA
+#pragma inline hot_msg_CreateDontCopy
+#endif
+MSG_INLINE
 hot_msg_t hot_msg_CreateDontCopy(void *data,
 				 unsigned nbytes,
 				 void (*on_release)(void*), 
@@ -559,6 +732,11 @@ hot_msg_t hot_msg_CreateDontCopy(void *data,
   ctl->need_release = 1;
   ctl->on_release = on_release;
   ctl->data = data;
+
+  /* The beginning of a message buffer must be 4-byte aligned.
+   */
+  assert(((unsigned)ctl->data) % 4 == 0);
+
   handle->offset = ctl->size = nbytes;
   ctl->writable = 0;
 
@@ -568,6 +746,10 @@ hot_msg_t hot_msg_CreateDontCopy(void *data,
 /* Write a buffer to the message.  The total size of the message is
  * limited to under 4K in the current implementation.
  */
+#ifdef INLINE_PRAGMA
+#pragma inline hot_msg_Write 
+#endif
+MSG_INLINE
 hot_err_t hot_msg_Write(hot_msg_t msg, void *data, unsigned nbytes) {
     struct hot_msg_handle *handle = (struct hot_msg_handle*) msg;
     struct hot_msg_ctl *ctl;
@@ -575,7 +757,7 @@ hot_err_t hot_msg_Write(hot_msg_t msg, void *data, unsigned nbytes) {
     hot_err_t err;
     
     /* Consistency check.
-   */
+     */
     if (handle == NULL || handle->magic != HOT_MSG_HANDLE_MAGIC)
 	return hot_err_Create(0, "bad handle");
     
@@ -622,6 +804,10 @@ hot_err_t hot_msg_Write(hot_msg_t msg, void *data, unsigned nbytes) {
  * specified buffer preallocated by the caller.
  * The message becomes read-only.
  */
+#ifdef INLINE_PRAGMA
+#pragma inline hot_msg_Read 
+#endif
+MSG_INLINE
 hot_err_t hot_msg_Read(hot_msg_t msg, void *buf, unsigned nbytes) {
     struct hot_msg_handle *handle = (struct hot_msg_handle*) msg;
     struct hot_msg_ctl *ctl;
@@ -640,17 +826,38 @@ hot_err_t hot_msg_Read(hot_msg_t msg, void *buf, unsigned nbytes) {
     if ((err = hot_lck_Lock(ctl->mutex)) != HOT_OK)
 	hot_sys_Panic(hot_err_ErrString(err));
     
-    /* Check for message boundary violations.
-     */
-    if (nbytes > handle->offset) {
-	result = hot_err_Create(0, "message boundary violation");
-	goto return_point;
-    }
-    
     /* Copy the data.  Message becomes read-only.
      */
-    handle->offset -= nbytes;
-    memcpy(buf, ctl->data + handle->offset, nbytes);
+    switch (handle->read_dir) {
+    case HOT_MSG_READ_LEFT:
+      /* Check for message boundary violations.
+       */
+      if (nbytes > handle->offset) {
+	result = hot_err_Create(0, "message boundary violation");
+	goto return_point;
+      }
+      
+      handle->offset -= nbytes;
+      memcpy(buf, ctl->data + handle->offset, nbytes);
+      break;
+
+    case HOT_MSG_READ_RIGHT:
+      /* Check for message boundary violations.
+       */
+      if (handle->offset + nbytes > ctl->size) {
+	result = hot_err_Create(0, "message boundary violation");
+	goto return_point;
+      }
+
+      memcpy(buf, ctl->data + handle->offset, nbytes);
+      handle->offset += nbytes;
+      break;
+
+    default:
+      result = hot_err_Create(0, "invalid read direction");
+      goto return_point;
+    }
+
     ctl->writable = 0;
     
   return_point:
@@ -665,6 +872,10 @@ hot_err_t hot_msg_Read(hot_msg_t msg, void *buf, unsigned nbytes) {
  * In case of success, pointer to data is returned in datap.  
  * The message becomes read-only.
  */
+#ifdef INLINE_PRAGMA
+#pragma inline hot_msg_Look 
+#endif
+MSG_INLINE
 hot_err_t hot_msg_Look(hot_msg_t msg, unsigned nbytes, /*OUT*/ void **datap) {
     struct hot_msg_handle *handle = (struct hot_msg_handle*) msg;
     struct hot_msg_ctl *ctl;
@@ -683,15 +894,36 @@ hot_err_t hot_msg_Look(hot_msg_t msg, unsigned nbytes, /*OUT*/ void **datap) {
     if ((err = hot_lck_Lock(ctl->mutex)) != HOT_OK)
 	hot_sys_Panic(hot_err_ErrString(err));
     
-    /* Check for message boundary violations.
-     */
-    if (nbytes > handle->offset) {
+    switch (handle->read_dir) {
+    case HOT_MSG_READ_LEFT:
+      /* Check for message boundary violations.
+       */
+      if (nbytes > handle->offset) {
 	result = hot_err_Create(0, "message boundary violation");
 	goto return_point;
+      }
+      
+      handle->offset -= nbytes;
+      *datap = ctl->data + handle->offset;
+      break;
+
+    case HOT_MSG_READ_RIGHT:
+      /* Check for message boundary violations.
+       */
+      if (handle->offset + nbytes > ctl->size) {
+	result = hot_err_Create(0, "message boundary violation");
+	goto return_point;
+      }
+
+      *datap = ctl->data + handle->offset;
+      handle->offset += nbytes;
+      break;
+
+    default:
+      result = hot_err_Create(0, "invalid read direction");
+      goto return_point;
     }
-    
-    handle->offset -= nbytes;
-    *datap = ctl->data + handle->offset;
+
     ctl->writable = 0;
     
   return_point:
@@ -699,8 +931,7 @@ hot_err_t hot_msg_Look(hot_msg_t msg, unsigned nbytes, /*OUT*/ void **datap) {
     if ((err = hot_lck_Unlock(ctl->mutex)) != HOT_OK)
 	hot_sys_Panic(hot_err_ErrString(err));
     
-    return result;
-    
+    return result;    
 }
 
 /* Pack messages into the specified destination.  
@@ -708,13 +939,18 @@ hot_err_t hot_msg_Look(hot_msg_t msg, unsigned nbytes, /*OUT*/ void **datap) {
  * Only tails of messages are packed (unread portions).  Note that the 
  * tail of a writeable message is the entire message.
  */
+#ifdef INLINE_PRAGMA
+#pragma inline hot_msg_Pack
+#endif
+MSG_INLINE
 hot_err_t hot_msg_Pack(hot_msg_t msg, hot_msg_t *msgs, unsigned nmsgs) {
     struct hot_msg_handle *handle = (struct hot_msg_handle*) msg;
     struct hot_msg_ctl *ctl;
     hot_err_t err = HOT_OK;
     unsigned magic = HOT_MSG_PACKED_MAGIC;
     unsigned new_size;
-    int i;
+    unsigned int i;
+    hot_uint32 tmp ;
     
     /* Consistency check.
      */
@@ -768,8 +1004,9 @@ hot_err_t hot_msg_Pack(hot_msg_t msg, hot_msg_t *msgs, unsigned nmsgs) {
 	 */
 	memcpy(ctl->data + ctl->size, m_ctl->data, m_handle->offset);
 	ctl->size += m_handle->offset;
-	memcpy(ctl->data + ctl->size, &m_handle->offset, 
-	       sizeof(m_handle->offset));
+	tmp = htonl(m_handle->offset) ;
+	memcpy(ctl->data + ctl->size, &tmp, 
+	       sizeof(tmp));
 	ctl->size += sizeof(m_handle->offset);    
       
 	if ((err = hot_lck_Unlock(m_ctl->mutex)) != HOT_OK)
@@ -778,10 +1015,12 @@ hot_err_t hot_msg_Pack(hot_msg_t msg, hot_msg_t *msgs, unsigned nmsgs) {
 
     /* Save the number of messages and the magic number.
      */
-    memcpy(ctl->data + ctl->size, &nmsgs, sizeof(nmsgs));
-    ctl->size += sizeof(nmsgs);
-    memcpy(ctl->data + ctl->size, &magic, sizeof(magic));
-    ctl->size += sizeof(magic);
+    tmp = htonl(nmsgs) ;
+    memcpy(ctl->data + ctl->size, &tmp, sizeof(tmp));
+    ctl->size += sizeof(tmp);
+    tmp = htonl(magic) ;
+    memcpy(ctl->data + ctl->size, &tmp, sizeof(tmp));
+    ctl->size += sizeof(tmp);
 
     handle->offset = ctl->size;	
 
@@ -797,17 +1036,24 @@ return_point:
  * In case of success, returns the pointer an array of unpacked 
  * messages in msgsp and array size in *nmsgsp.
  */
+#ifdef INLINE_PRAGMA
+#pragma inline hot_msg_Unpack
+#endif
+MSG_INLINE
 hot_err_t hot_msg_Unpack(hot_msg_t msg, /*OUT*/ hot_msg_t **msgsp, 
 			 /*OUT*/ unsigned *nmsgsp) {
     struct hot_msg_handle *handle = (struct hot_msg_handle*) msg;
     struct hot_msg_ctl *ctl;
     struct hot_msg_handle **msgs = NULL ; /* to avoid -Wall warning */
-    unsigned nmsgs;
+    unsigned nmsgs = 0 ;	/* to avoid -Wall warning */
     hot_err_t result = HOT_OK;
-    int i, cancel = 0;
+    unsigned int i ;
+    int cancel = 0;
     unsigned magic;
     hot_err_t err;
-    
+    hot_uint32 tmp ;
+    int j;
+
     /* Consistency check.
      */
     if (handle == NULL || handle->magic != HOT_MSG_HANDLE_MAGIC)
@@ -829,7 +1075,8 @@ hot_err_t hot_msg_Unpack(hot_msg_t msg, /*OUT*/ hot_msg_t **msgsp,
     
     /* Do some "typechecking".
      */
-    memcpy(&magic, ctl->data + (handle->offset -= sizeof(magic)), sizeof(magic));
+    memcpy(&tmp, ctl->data + (handle->offset -= sizeof(tmp)), sizeof(tmp));
+    magic = ntohl(tmp) ;
     if (magic != HOT_MSG_PACKED_MAGIC) {
 	result = hot_err_Create(0, "message is not packed");
 	handle->offset += sizeof(magic);
@@ -838,18 +1085,21 @@ hot_err_t hot_msg_Unpack(hot_msg_t msg, /*OUT*/ hot_msg_t **msgsp,
     
     /* Allocate space for included messages.
      */
-    memcpy(&nmsgs, ctl->data + (handle->offset -= sizeof(nmsgs)), sizeof(nmsgs));
+    memcpy(&tmp, ctl->data + (handle->offset -= sizeof(tmp)), sizeof(tmp));
+    nmsgs = ntohl(tmp); 
     msgs = (struct hot_msg_handle**) 
-      malloc(nmsgs * sizeof(struct hot_msg_handle*));
-    if (msgs == NULL)
+      hot_mem_Alloc(global.memory, nmsgs * sizeof(struct hot_msg_handle*));
+    if (msgs == NULL) {
 	hot_sys_Panic("hot_msg_Unpack:  malloc");
-    for (i = 0; i < nmsgs; i++)
+    }
+    for (i = 0; i < nmsgs; i++) {
 	msgs[i] = (struct hot_msg_handle*) hot_msg_Create();
+    }
     
     /* Fill in the messages.
      */
-    for (i = nmsgs - 1; i >= 0; i--) {
-	struct hot_msg_handle *m_handle = msgs[i];
+    for (j = nmsgs - 1; j >= 0; j--) {
+	struct hot_msg_handle *m_handle = msgs[j];
 	struct hot_msg_ctl *m_ctl = m_handle->ctl;
 	
 	/* Consistency check.
@@ -862,8 +1112,10 @@ hot_err_t hot_msg_Unpack(hot_msg_t msg, /*OUT*/ hot_msg_t **msgsp,
 	
 	/* Get the size of the msg.
 	 */
-	memcpy(&m_ctl->size, ctl->data + (handle->offset -= sizeof(m_ctl->size)),
-	       sizeof(m_ctl->size));
+	memcpy(&tmp, ctl->data + (handle->offset -= sizeof(tmp)),
+	       sizeof(tmp));
+	m_ctl->size = ntohl(tmp) ;
+
 	m_handle->offset = m_ctl->size;
 	
 	/* Consistency check.
@@ -890,7 +1142,7 @@ return_point:
 	 */
 	for (i = 0; i < nmsgs; i++)
 	    hot_msg_Release((hot_msg_t) msgs[i]);
-	free(msgs);
+	hot_mem_Free(msgs);
     }
     
     if ((err = hot_lck_Unlock(ctl->mutex)) != HOT_OK)
@@ -901,6 +1153,10 @@ return_point:
 
 /* Returns non-zero iff the message is writable.
  */
+#ifdef INLINE_PRAGMA
+#pragma inline hot_msg_IsWritable
+#endif
+MSG_INLINE
 int hot_msg_IsWritable(hot_msg_t msg) {
   struct hot_msg_handle *handle = (struct hot_msg_handle*) msg;
   struct hot_msg_ctl *ctl;
@@ -915,6 +1171,10 @@ int hot_msg_IsWritable(hot_msg_t msg) {
 
 /* Write an unsigned 32-bit integer to the message.  
  */
+#ifdef INLINE_PRAGMA
+#pragma inline hot_msg_WriteUint32
+#endif
+MSG_INLINE
 hot_err_t hot_msg_WriteUint32(hot_msg_t msg, hot_uint32 *u) {
     hot_uint32 net;
     net = htonl(*u);
@@ -923,6 +1183,10 @@ hot_err_t hot_msg_WriteUint32(hot_msg_t msg, hot_uint32 *u) {
 
 /* Read an unsigned 32-bit integer from the message.  
  */
+#ifdef INLINE_PRAGMA
+#pragma inline hot_msg_ReadUint32
+#endif
+MSG_INLINE
 hot_err_t hot_msg_ReadUint32(hot_msg_t msg, hot_uint32 *u) {
     hot_uint32 net;
     hot_err_t err = hot_msg_Read(msg, &net, 4);
