@@ -1,7 +1,7 @@
 /**************************************************************/
 /*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -14,6 +14,9 @@
  *
  */
 
+#ifndef __HOT_ERROR_H__
+#define __HOT_ERROR_H__
+
 #include <string.h>
 #include <memory.h>
 #include <malloc.h>
@@ -29,7 +32,7 @@ typedef struct hot_err *hot_err_t ;
 
 /* Create an error object with specified code and reason
  */ 
-hot_err_t hot_err_Create(unsigned err_code, char *err_string);
+hot_err_t hot_err_Create(unsigned err_code, const char *err_string);
 
 /* Release an error object
  */
@@ -47,14 +50,12 @@ unsigned hot_err_ErrCode(hot_err_t err);
  */
 void hot_err_Panic(hot_err_t err, char *info) ;
 
-/*#define HOT_ERROR_DEBUG_MEMORY_LEAKS*/
+#define hot_err_Check(err)                      \
+  do {                                          \
+    hot_err_t _hot_err_hide = (err) ;           \
+    if (_hot_err_hide != HOT_OK)                \
+      hot_sys_Panic(hot_err_ErrString(_hot_err_hide)) ; \
+  } while (0)
 
-#if (defined HOT_ERROR_DEBUG_MEMORY_LEAKS) && (!defined HOT_ERROR_NO_DEBUG_WRAPPERS)
+#endif /*__HOT_ERROR_H__*/
 
-char *hot_debug_mem_Malloc(unsigned size, char *file, unsigned line);
-void hot_debug_mem_Free(void *ptr, char *file, unsigned line);
-
-#define malloc(s) hot_debug_mem_Malloc((s),__FILE__,__LINE__)
-#define free(p) hot_debug_mem_Free((p),__FILE__,__LINE__)
-
-#endif
