@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -12,8 +12,16 @@
 (* Author: Mark Hayden, 4/97 *)
 (**************************************************************)
 
-(* Async_add: returns disable function.
+type t
+
+val create : Sched.t -> t
+
+(* Async.add: called by "receiver" to register a callback.
+ * It returns disable function.
  *)
-val add : (Group.id * Endpt.id) -> (unit -> unit) -> (unit -> unit) 
-val find : (Group.id * Endpt.id) -> (unit -> unit)
+val add : t -> (Group.id * Endpt.id) -> (unit -> unit) -> (unit -> unit) 
+
+(* Find is called by "sender" to generate new callback.
+ *)
+val find : t -> (Group.id * Endpt.id) -> (unit -> unit)
 
