@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -13,7 +13,8 @@
 (**************************************************************)
 
 val f :
-  ('local,'abv,'hdr) Layer.msg ->
-  View.local * View.state ->
+  Alarm.t ->
+  (Addr.id -> int) ->
+  ('local,'abv,'hdr) Layer.msg -> View.full ->
   (Event.t -> ('local,'abv,'hdr) Layer.msg -> unit) ->
   (Event.t -> ('local,'abv,'hdr) Layer.msg -> unit)
