@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -17,20 +17,24 @@ open Util
 open View
 open Event
 (**************************************************************)
+let name = Trace.file "DISP"
+(**************************************************************)
 
-let l name (ls,vs) = ((),fun {up_lout=up;dn_lout=dn} ->
+let l _ (ls,vs) = ((),fun {up_lout=up;dn_lout=dn} ->
   let msg = sprintf "%s{%s}:" name (Endpt.string_of_id ls.endpt) in
 
   let up_hdlr ev hdr =
     if getType ev <> ETimer then (
-      printf "%s%s" msg (to_string ev) ;
-      up ev hdr
-    )
+      eprintf "%sUp(%s)\n" msg (to_string ev) ;
+    ) ;
+    up ev hdr
   and dn_hdlr ev hdr =
     if getType ev <> ETimer then (
-      printf "%s%s" msg (to_string ev) ;
-      dn ev hdr
-    )
-  in {up_lin=up;dn_lin=dn})
+      eprintf "%sDn(%s)\n" msg (to_string ev) ;
+    ) ;
+    dn ev hdr
+  in {up_lin=up_hdlr;dn_lin=dn_hdlr})
+
+let _ = Elink.layer_install "DISP" l
 
 (**************************************************************)
