@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -13,4 +13,4 @@
 (* Based on code by Werner Vogels *)
 (**************************************************************)
 
-val l : string -> ('a,'a,unit) Layer.basic
+val l : ('a,'a,unit) Layer.basic
