@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -12,14 +12,11 @@
 (* Author: Mark Hayden, 4/96 *)
 (**************************************************************)
 
-(* Top-level scheduler.
- *)
-val root_sched : Sched.t
-
-(**************************************************************)
-
 val config_full : 
-  Appl_intf.t ->			(* application *)
+  Glue.glue ->				(* glue *)
+  Alarm.t ->
+  (Addr.id -> int) ->
+  Layer.state ->			(* application *)
   View.full ->				(* view state *)
   (Event.up -> unit) ->			(* events out of top *)
   (Event.dn -> unit)			(* events into top *)
@@ -27,7 +24,10 @@ val config_full :
 (**************************************************************)
 
 val config :
-  Appl_intf.t ->			(* application *)
+  Glue.glue ->				(* glue *)
+  Alarm.t ->
+  (Addr.id -> int) ->
+  Layer.state ->			(* application *)
   View.full ->				(* view state *)
   unit
 
