@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -30,7 +30,7 @@ open Event
 open Util
 open View
 (**************************************************************)
-let name = Trace.source_file "DELAY"
+let name = Trace.filel "DELAY"
 (**************************************************************)
 
 type header = NoHdr
@@ -42,19 +42,20 @@ type state = {
 
 (**************************************************************)
 
-let dump (ls,vs) s = ()
+let dump = ignore2
 let debug = false
 
 (**************************************************************)
 
-let init () (ls,vs) = { 
+let init _ (ls,vs) = { 
   counter  = 12 ;
   count    = 0
 }
   
+(**************************************************************)
 
-let hdlrs s (ls,vs) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dnnm} =
-  let failwith m = dump (ls,vs) s ; failwith (name^":"^m) in
+let hdlrs s ((ls,vs) as vf) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dnnm} =
+  let failwith = layer_fail dump vf s name in
 
   let up_hdlr ev abv hdr = match getType ev, hdr with
   | ECast, NoHdr -> 
@@ -68,17 +69,10 @@ let hdlrs s (ls,vs) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dn
   | _, NoHdr -> up ev abv 
 
   and uplm_hdlr ev hdr = match getType ev,hdr with
-  | _ -> failwith "unknown local message"
+  | _ -> failwith unknown_local
 
   and upnm_hdlr ev = match getType ev with
-    (* EInit: Initialize timer.
-     *)
-  | EInit -> 
-      let seed = truncate (Time.to_float (getTime ev)) in 
-      if debug then printf "[DELAY seed=%d]" seed;
-      Random.init (seed);
-      upnm ev  
-  | EDump -> ( dump (ls,vs) s ; upnm ev ) 
+  | EDump -> ( dump vf s ; upnm ev ) 
   | _ -> upnm ev
 
   and dn_hdlr ev abv = match getType ev with
@@ -88,9 +82,9 @@ let hdlrs s (ls,vs) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dn
 
 in {up_in=up_hdlr;uplm_in=uplm_hdlr;upnm_in=upnm_hdlr;dn_in=dn_hdlr;dnnm_in=dnnm_hdlr}
 
-let l args vs = Layer.hdr_state init hdlrs None NoOpt args vs
+let l args vs = Layer.hdr init hdlrs None NoOpt args vs
 
-let _ = Layer.install name (Layer.init l)
+let _ = Elink.layer_install name l
 
 (**************************************************************)
 
