@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -25,8 +25,8 @@ open Event
 open Util
 open Trans
 (**************************************************************)
-let name = Trace.source_file "SEQUENCER"
-let failwith s = failwith (Util.failmsg name s)
+let name = Trace.filel "SEQUENCER"
+let failwith = Trace.make_failwith name
 (**************************************************************)
 (* Headers
 
@@ -48,98 +48,87 @@ type header = NoHdr
 
 (**************************************************************)
 
-type 'abv item =
-  | Msg of 'abv * Iovecl.t
-  | Unset				(* for iq *)
-  | Reset				(* for iq *)
-
 type ('abv,'b) state = {
-  seq_rank		: int ;         (* who is the sequencer *)
-  ordered		: int array ;
+  seq_rank		: rank ;        (* who is the sequencer *)
+  ordered		: seqno array ;
   mutable got_view	: bool ;	(* have we got an EView? *)
   mutable blocking 	: bool ;	(* are we blocking? *)
 
   (* buffered send messages (waiting for delivery from coordinator) *)
-  casts 		: 'abv item Iq.t ;
+  casts 		: 'abv Iq.t ;
 
   (* buffered recv messages waiting for the end of the view *)
-  up_unord 		: (seqno * 'abv * Iovecl.t)  Queue.t array
+  up_unord 		: (seqno * 'abv * Iovecl.t)  Queuee.t array
 }
 
 (**************************************************************)
 
 let dump (ls,vs) s =
   eprintf "SEQUENCER:dump:%s\n" ls.name ;
-  eprintf " blocking=%b \n" s.blocking
+  sprintf "blocking=%b\n" s.blocking
 
 (**************************************************************)
 
-let init () (ls,vs) = {
+let init _ (ls,vs) = {
   seq_rank      = 0 ;			(* always rank 0 *)
-  ordered	= array_create name ls.nmembers 0 ;
+  ordered	= Array.create ls.nmembers 0 ;
   blocking 	= false ;
-  casts 	= Iq.empty Unset Reset ;
+  casts 	= Iq.create name NoMsg ;
   got_view	= false ;
   up_unord 	=
-    Array.map (fun _ ->
-      Queue.create ()
-    ) vs.view
+    Array.init ls.nmembers (fun _ -> Queuee.create ())
 }
 
 (**************************************************************)
 
-let hdlrs s (ls,vs) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dnnm} =
-  let ack = make_acker name dnnm in
-
-  (* Called when blocking is about to begin.  Send any
-   * messages we have buffered as 'unordered.'
-   *)
-  let now_blocking () =
-    if not s.blocking then (
-      s.blocking <- true ;
-      List.iter (function (seqno,Msg(abv,iov)) ->
-	dn (castIov name iov) abv (Unordered seqno) ;
-	Queue.add (seqno,abv,iov) s.up_unord.(ls.rank)
-      | _ -> failwith "buffer sanity"
-      ) (Iq.list_of_iq s.casts)
-    )
-  in
+let hdlrs s ((ls,vs) as vf) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dnnm} =
+  let log = Trace.log2 name ls.name in
 
   let up_hdlr ev abv hdr = match getType ev, hdr with
     | ESend, ToSeq ->
-	if (ls.rank <> s.seq_rank) then
+	if ls.rank <> s.seq_rank then
 	  failwith "send when not sequencer" ;
 
 	(* Bounce the message back out and deliver locally. 
 	 *)
 	if not s.blocking then (
-	  up (castOriginIov name (getOrigin ev) (getIov ev)) abv ;
-	  if ls.nmembers > 1 then
-	    dn (castIov name (getIov ev)) abv (Ordered (getOrigin ev)) ;
-	  s.ordered.((getOrigin ev)) <- succ s.ordered.((getOrigin ev))
+	  let origin = getPeer ev in
+	  let iov = getIov ev in
+	  let iov' = Iovecl.copy "Sequ:local" iov in
+	  up (castPeerIov name origin iov') abv ;
+	  if ls.nmembers > 1 then (
+	    let iov' = Iovecl.copy "Sequ:bounce" iov in
+	    dn (castIovAppl name iov') abv (Ordered origin)
+	  ) ;
+	  array_incr s.ordered origin ;
 	) ;
-	ack ev ; free name ev
+	free name ev
 
-    | ECast, Ordered(rank) ->
+    | ECast, Ordered rank ->
       	if s.got_view then
 	  failwith "got ordered after view" ;
-	  
+	if s.seq_rank = ls.rank then
+	  failwith "sequencer got ordered message" ;
+
       	(* If it's from me, then remove from my iq.
 	 *)
 	if rank = ls.rank then
-	  Iq.advance_head s.casts s.ordered.(rank) ;
+	  Iq.set_lo s.casts s.ordered.(rank) ;
 
       	(* Deliver ordered messages.  Keep track of how many
 	 * I've got from each member.
        	 *)
-	s.ordered.(rank) <- succ s.ordered.(rank) ;
-	up (set name ev[Origin rank]) abv
+(*	log (fun () -> sprintf "delivering ordered msg from %d: #ordered=%d" rank s.ordered.(rank)) ;*)
+	array_incr s.ordered rank ;
+	up (set name ev[Peer rank]) abv
 
-    | ECast, Unordered(seqno) ->
+    | ECast, Unordered seqno ->
       	(* Buffer unordered messages until the view.
 	 *)
-	Queue.add (seqno,abv,(getIov ev)) s.up_unord.((getOrigin ev)) ;
-	ack ev ; free name ev
+	let iov = Iovecl.copy "Sequ:Unord" (getIov ev) in
+	Queuee.add (seqno,abv,iov) s.up_unord.(getPeer ev) ;
+	free name ev
+
     | _, NoHdr      -> up ev abv (* all other events have NoHdr *)
     | _             -> failwith "non-NoHdr on non ECast"
 
@@ -152,9 +141,13 @@ let hdlrs s (ls,vs) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dn
       (* Deliver unordered messages and then the view.
        *)
       for i = 0 to pred ls.nmembers do
-	queue_clean (fun (seqno,abv,iov) ->
-	  if seqno >= s.ordered.(i) then
-	    up (castOriginIov name i iov) abv
+	Queuee.clean (fun (seqno,abv,iov) ->
+	  if seqno >= s.ordered.(i) then (
+(*	    log (fun () -> sprintf "delivering unordered msg:seqno=%d, #ordered=%d" seqno s.ordered.(i)) ;*)
+	    up (castPeerIov name i iov) abv
+	  ) else (
+	    Iovecl.free name iov
+	  )
 	) s.up_unord.(i)
       done ;
       upnm ev
@@ -162,6 +155,8 @@ let hdlrs s (ls,vs) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dn
 
   and dn_hdlr ev abv = match getType ev with
   | ECast ->
+      let iov = getIov ev in
+
       if getNoTotal ev then (
       	(* Send without ordering properties.
 	 *)
@@ -174,25 +169,28 @@ let hdlrs s (ls,vs) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dn
 	 * Otherwise, send pt2pt to sequencer and stash a copy of it in
 	 * case the sequencer fails and we need to resend it.
 	 *)
+	let iov = Iovecl.copy "Sequ:coord" (getIov ev) in
 	if ls.nmembers > 1 then
-	  dn ev abv (Ordered(ls.rank)) ;
-	up (castOriginIov name ls.rank (getIov ev)) abv ;
-	s.ordered.(ls.rank) <- succ s.ordered.(ls.rank)
+	  dn ev abv (Ordered(ls.rank))
+	else
+	  free name ev ;
+	up (castPeerIov name ls.rank iov) abv ;
+	array_incr s.ordered ls.rank
       ) else (
-	Iq.add s.casts (Msg(abv,(getIov ev))) ;
-	dn (sendRanksIov name [s.seq_rank] (getIov ev)) abv ToSeq ;
+	ignore (Iq.add s.casts iov abv) ;
+	let iov = Iovecl.copy "Sequ:send_to_coord" iov in
+	dn (sendPeerIov name s.seq_rank iov) abv ToSeq ;
 	free name ev
       )
 
     (* Handle local delivery for sends.
      *)
   | ESend ->
-      if List.mem ls.rank (getRanks ev) then (
-      	up (create name ESend[Origin ls.rank; Iov (getIov ev)]) abv ;
-	if List.length (getRanks ev) = 1 then
-  	  free name ev
-	else
-	  dn (set name ev[Ranks (except ls.rank (getRanks ev))]) abv NoHdr
+      let dest = getPeer ev in
+      if dest = ls.rank then (
+	let iov = Iovecl.copy "Seqno:local_send" (getIov ev) in
+      	up (sendPeerIov name ls.rank iov) abv ;
+  	free name ev
       ) else (
 	dn ev abv NoHdr
       )
@@ -200,17 +198,25 @@ let hdlrs s (ls,vs) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dn
   | _ -> dn ev abv NoHdr
 
   and dnnm_hdlr ev = match getType ev with
-  | EBlockOk ->
-      now_blocking () ;
+    (* Send any messages we have buffered as 'unordered.'  
+     *)
+  | EBlock ->
+      if not s.blocking then (
+	s.blocking <- true ;
+	List.iter (fun (seqno,iov,abv) ->
+	  Queuee.add (seqno,abv,iov) s.up_unord.(ls.rank) ;
+	  let iov = Iovecl.copy "Sequ:now_blocking" iov in
+	  dn (castIovAppl name iov) abv (Unordered seqno)
+	) (Iq.list_of_iq s.casts)
+      ) ;
       dnnm ev
-  | EAck ->
-      free name ev
+
   | _ -> dnnm ev
 
 in {up_in=up_hdlr;uplm_in=uplm_hdlr;upnm_in=upnm_hdlr;dn_in=dn_hdlr;dnnm_in=dnnm_hdlr}
 
 let l args vf = Layer.hdr init hdlrs None NoOpt args vf
 
-let _ = Layer.install name (Layer.init l)
+let _ = Elink.layer_install name l
 
 (**************************************************************)
