@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -37,9 +37,8 @@ open Event
 open Util
 open View
 (**************************************************************)
-let name = Trace.source_file "TOTEM"
-let failwith s = failwith (Util.failmsg name s)
-(*let verbose = ref true*)
+let name = Trace.filel "TOTEM"
+let failwith = Trace.make_failwith name
 (**************************************************************)
 (* Headers
 
@@ -70,34 +69,27 @@ type header = NoHdr
 
 (**************************************************************)
 
-type 'abv item =
-  | Item of rank * 'abv * Iovecl.t
-  | Unset				(* for iq *)
-  | Reset				(* for iq *)
-
-(**************************************************************)
-
 type 'abv state = {
   prev		: rank ;		(* prev member in cycle *)
   next		: rank ;		(* next member in cycle *)
   mutable got_view : bool ;
   mutable blocking : bool ;		(* are we blocking? *)
-  waiting 	: ('abv * Iovecl.t) Queue.t ;(* waiting for token *)
-  order 	: 'abv item Iq.t ;	(* ordered by token *)
-  unord 	: ('abv * Iovecl.t) Queue.t array ; (* unordered *)
+  waiting 	: ('abv * Iovecl.t) Queuee.t ;(* waiting for token *)
+  order 	: (rank * 'abv) Iq.t ;	(* ordered by token *)
+  unord 	: ('abv * Iovecl.t) Queuee.t array ; (* unordered *)
   mutable token	: seqno option
 }
 
 (**************************************************************)
 
-let dump (ls,vs) s =
-  eprintf "TOTEM:dump:%s\n" ls.name ;
-  eprintf " blocking=%b \n" s.blocking ;
-  eprintf " waiting=%d\n" (Queue.length s.waiting) ;
-  eprintf " order=%d..%d\n" (Iq.head s.order) (Iq.tail s.order) ;
+let dump vf s = Layer.layer_dump name (fun (ls,vs) s -> [|
+  sprintf "blocking=%b \n" s.blocking ;
+  sprintf "waiting=%d\n" (Queuee.length s.waiting) ;
+  sprintf "order=%d..%d\n" (Iq.lo s.order) (Iq.hi s.order) ;
   let l = Iq.list_of_iq s.order in
-  let l = string_of_list (function (s,Item(r,_,_)) -> sprintf "(%d,%d)" r s | _ -> failwith "sanity") l in
-  eprintf "   =%s\n" l
+  let l = string_of_list (fun (s,_,(r,_)) -> sprintf "(%d,%d)" r s) l in
+  sprintf "=%s\n" l
+|]) vf s
 
 (**************************************************************)
 
@@ -106,17 +98,17 @@ let init await_ack (ls,vs) = {
   blocking 	= false ;
   next		= (succ ls.rank) mod ls.nmembers ;
   prev		= (pred ls.rank + ls.nmembers) mod ls.nmembers ;(* Hack! *)
-  waiting 	= Queue.create () ;
-  order 	= Iq.empty Unset Reset ;
+  waiting 	= Queuee.create () ;
+  order 	= Iq.create name (0,NoMsg) ;
   token		= None ;
-  unord 	= array_createf ls.nmembers (fun _ -> Queue.create ())
+  unord 	= Array.init ls.nmembers (fun _ -> Queuee.create ())
 }
 
 (**************************************************************)
 
-let hdlrs s (ls,vs) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dnnm} =
-  let ack = make_acker name dnnm in
-  let failwith m = dump (ls,vs) s ; failwith m in
+let hdlrs s ((ls,vs) as vf) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dnnm} =
+(*let ack = make_acker name dnnm in*)
+  let failwith m = dump vf s ; failwith m in
 
   (* Check for any requests.  If there are requests and we
    * have no messages buffered, then just send the token on.
@@ -125,21 +117,21 @@ let hdlrs s (ls,vs) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dn
    *)
   let check_token () =
     if_some s.token (fun token ->
-      if Iq.head s.order >= token then (
+      if Iq.lo s.order >= token then (
 	s.token <- None ;
-	let len = Queue.length s.waiting in
+	let len = Queuee.length s.waiting in
 	if len = 0 then (
-	  dnlm (sendRank name s.next) (TokenSend(token))
+	  dnlm (sendPeer name s.next) (TokenSend(token))
 	) else (
 	  if !verbose then
       	    eprintf "TOTEM:%d->[%d..%d]\n" ls.rank token (token+len-1) ;
 	  for i = 0 to pred len do
-	    let abv,iov = Queue.take s.waiting in
+	    let abv,iov = Queuee.take s.waiting in
 	    let pass = (i = pred len) in
 	    dn (castIov name iov) abv (Ordered(token + i,pass)) ;
-	    up (castOriginIov name ls.rank iov) abv ;
-	    if not (Iq.opt_check_update s.order (token + i)) then
-	      failwith "sanity"
+	    up (castPeerIov name ls.rank iov) abv ;
+	    if not (Iq.opt_update_old s.order (token + i)) then
+	      failwith sanity
 	  done
 	)
       )
@@ -159,35 +151,31 @@ let hdlrs s (ls,vs) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dn
       	failwith "ECast(Ordered) after EView" ;
       if !verbose then
 	eprintf "TOTEM:%d:Ordered:seqno=%d\n" ls.rank seqno ;
-      if Iq.opt_check_update s.order seqno then (
-	if !verbose then
-      	  eprintf "  (opt_check_update)\n" ;
+      if Iq.opt_update_old s.order seqno then (
         up ev abv
       ) else (
-	if not (Iq.assign s.order seqno (Item((getOrigin ev),abv,(getIov ev)))) then
+	let origin = getPeer ev in
+	let iov = getIov ev in
+	if not (Iq.assign s.order seqno iov (origin,abv)) then
 	  failwith "2nd seqno of same seqno" ;
-	Iq.get_prefix s.order (fun seqno -> function
-	  | Item(origin,abv,iov) ->
-	      if !verbose then
-		eprintf "  (delivering:%d)\n" seqno ;
-	      up (castOriginIov name origin iov) abv
-	  | _ -> failwith "iqsanity"
+	Iq.get_prefix s.order (fun seqno iov (origin,abv) ->
+	  up (castPeerIov name origin iov) abv
 	) ;
-        ack ev ; free name ev
+        (*ack ev ;*) free name ev
       ) ;
 
       check_token () ;
 
       (* Check if the token is for me.
        *)
-      if token & (getOrigin ev) = s.prev then
+      if token & (getPeer ev) = s.prev then
         got_token (succ seqno)
 
   | ECast, Unordered ->
       if s.got_view then
       	failwith "ECast(Unordered) after EView" ;
-      Queue.add (abv,(getIov ev)) s.unord.((getOrigin ev)) ;
-      ack ev ; free name ev
+      Queuee.add (abv,(getIov ev)) s.unord.((getPeer ev)) ;
+      (*ack ev ;*) free name ev
 
   | _, NoHdr      -> up ev abv (* all other events have NoHdr *)
   | _             -> failwith "non-NoHdr on non ECast"
@@ -195,7 +183,7 @@ let hdlrs s (ls,vs) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dn
   and uplm_hdlr ev hdr = match getType ev,hdr with
   | ESend, TokenSend(token) ->
       got_token token ;
-      ack ev ; free name ev
+      (*ack ev ;*) free name ev
 
   | _ -> failwith "non-NoHdr on non ECast"
 
@@ -203,17 +191,17 @@ let hdlrs s (ls,vs) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dn
   | EInit ->
       upnm ev;
       if ls.rank = 0 && ls.nmembers > 1 then
-	dnlm (sendRank name s.next) (TokenSend(0))
+	dnlm (sendPeer name s.next) (TokenSend(0))
   | EView ->
       s.got_view <- true ;
 
       (* Deliver unordered messages.
        *)
       for i = 0 to pred ls.nmembers do
-        queue_clean (fun (abv,iov) ->
+        Queuee.clean (fun (abv,iov) ->
 	  if !verbose then
 	    eprintf "TOTEM:unordered:%d->%d\n" i ls.rank ;
-          up (castOriginIov name i iov) abv
+          up (castPeerIov name i iov) abv
         ) s.unord.(i)
       done ;
       upnm ev
@@ -226,30 +214,31 @@ let hdlrs s (ls,vs) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dn
       ) else if ls.nmembers > 1 then (
 	if s.got_view then
 	  failwith "ECast after EView" ;
-	Queue.add (abv,(getIov ev)) s.waiting ;
+	Queuee.add (abv,(getIov ev)) s.waiting ;
         free name ev
       ) else (
-	up (castOriginIov name ls.rank (getIov ev)) abv ;
+	up (castPeerIov name ls.rank (getIov ev)) abv ;
         free name ev
       )
 
   | _ -> dn ev abv NoHdr
 
   and dnnm_hdlr ev = match getType ev with
-  | EBlockOk ->
+  | EBlock ->
       if s.blocking then 
       	failwith "blocking twice" ;
       s.blocking <- true ;
       if !verbose then
-	eprintf "TOTEM:sending %d unordered\n" (Queue.length s.waiting) ;
-      Util.queue_clean (fun (abv,iov) ->
-      	Queue.add (abv,iov) s.unord.(ls.rank) ;
+	eprintf "TOTEM:sending %d unordered\n" (Queuee.length s.waiting) ;
+      Queuee.clean (fun (abv,iov) ->
+      	Queuee.add (abv,iov) s.unord.(ls.rank) ;
 	dn (castIov name iov) abv Unordered
       ) s.waiting ;
       dnnm ev
-
+(*
   | EAck ->
       free name ev
+*)
 
   | _ -> dnnm ev
 
@@ -257,6 +246,6 @@ in {up_in=up_hdlr;uplm_in=uplm_hdlr;upnm_in=upnm_hdlr;dn_in=dn_hdlr;dnnm_in=dnnm
 
 let l args vf = Layer.hdr init hdlrs None NoOpt args vf
 
-let _ = Layer.install name (Layer.init l)
+let _ = Elink.layer_install name l
 
 (**************************************************************)
