@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -12,6 +12,4 @@
 (* Author: Mark Hayden, 3/97 *)
 (**************************************************************)
 
-val f : (Iovecl.t -> unit) Route.t
-
-(*val f : Route.message Route.t*)
+val f : Mbuf.t -> (Iovecl.t -> unit) Route.t
