@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -12,4 +12,4 @@
 (* Author: Mark Hayden, 3/97 *)
 (**************************************************************)
 
-val f : (Obj.t option -> Iovecl.t -> unit) Route.t
+val f : Mbuf.t -> (Trans.rank -> Obj.t option -> int -> Iovecl.t -> unit) Route.t
