@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -12,7 +12,7 @@
 (* Author: Robbert vanRenesse *)
 (**************************************************************)
 val half_time : float
-type availability = UP | DOWN of float
+type availability = UP | DOWN of Time.t
 type server_info =
   { mutable avail: availability;
     mutable conf_value: float;
