@@ -1,7 +1,7 @@
 #*************************************************************#
 #
-#   Ensemble, (Version 0.40)
-#   Copyright 1997 Cornell University
+#   Ensemble, (Version 0.70p1)
+#   Copyright 2000 Cornell University
 #   All rights reserved.
 #
 #   See ensemble/doc/license.txt for further information.
@@ -16,6 +16,6 @@ export MACHTYPE
 
 ( cd opt ; make )
 ( cd opt ; make socket )
-( cd opt ; make libhot )
+( cd opt ; make hot )
 ( cd maestro ; make )
 ( cd demo ; make socketopt )
