@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -12,4 +12,5 @@
 (* Author: Mark Hayden, 3/96 *)
 (**************************************************************)
 
-val install_blocker : (float -> unit) -> unit
+val export_socks : Hsys.socket Arrayf.t ref
+val install_blocker : (Time.t -> unit) -> unit
