@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -11,25 +11,25 @@
 (* GROUP.ML *)
 (* Author: Mark Hayden, 4/96 *)
 (**************************************************************)
-let name = Trace.source_file "GROUP"
-let failwith s = failwith (Util.failmsg name s)
-(**************************************************************)
 open Util
 (**************************************************************)
+let name = Trace.file "GROUP"
+let failwith = Trace.make_failwith name
+(**************************************************************)
 
 type id	= 
   | Anon of Unique.id			(* anonymous groups *)
   | Named of string			(* named groups *)
 
-let id () = Anon (Unique.id ())
+let id u = Anon u
 let named name = Named name
 
 let string_of_id = function
-  | Named name -> sprintf "{Group:Named:%s}" name
   | Anon id    -> sprintf "{Group:Anon:%s}" (Unique.string_of_id_short id)
+  | Named name -> sprintf "{Group:Named:%s}" name
 
 let hash_of_id = function
-  | Named name -> Hashtbl.hash (Digest.string name)
   | Anon id -> Unique.hash_of_id id
+  | Named name -> Hashtbl.hash (Digest.string name)
 
 (**************************************************************)
