@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -18,7 +18,7 @@ type id
 
 (* Constructors.
  *)
-val id    : unit -> id			(* anonymous groups *)
+val id    : Unique.id -> id		(* anonymous groups *)
 val named : string -> id		(* named groups *)
 
 (* Display function.
