@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -12,6 +12,11 @@
 (* Author: Mark Hayden, 4/95 *)
 (**************************************************************)
 open Util
+open Tdefs
+open Trans
+
+(**************************************************************)
+let name = Trace.file "LAYER"
 (**************************************************************)
 
 type 'a saved = 'a option ref
@@ -42,13 +47,68 @@ type ('local,'hdr,'abv) full =
 (* What messages look like in this architecture.
  *)
 type ('local,'hdr,'abv) msg =
+  (* This comes first in order to enable some hacks.
+   *)
+  | Local_nohdr
+  | Local_seqno of Trans.seqno
+
   | NoMsg
   | Local of 'local
   | Full of 'hdr * 'abv
   | Full_nohdr of 'abv
-  | Local_nohdr
-  | Local_seqno of Trans.seqno
+
+let string_of_msg local hdr abv = function
+  | Local_nohdr -> "Local_nohdr"
+  | Local_seqno seqno -> sprintf "Local_seqno(%d)" seqno
+  | NoMsg -> "NoMsg"
+  | Local a -> sprintf "Local(%s)" (local a)
+  | Full(a,b) -> sprintf "Full(%s,%s)" (hdr a) (abv b)
+  | Full_nohdr a -> sprintf "Full_nohdr(%s)" (abv a)
+
+external local_seqno_coerce  : ('a,'b,'c)msg -> ('d,'e,'f)msg = "%identity"
+
+let local_seqno_of_some = function
+  | (Some _) as s -> (Obj.magic s : ('a,'b,'c)msg)
+  | None -> failwith sanity
+
+let option_of_local_seqno = function
+  | (Local_seqno _) as s -> (Obj.magic s : Trans.seqno option)
+  | _ -> failwith sanity
     
+(**************************************************************)
+(* Record containing overall fields for protocol stacks.
+ * Assumes no layers are repeated.  
+ *)
+type state = {
+  interface        : Appl_intf.New.t ;
+  switch	   : Time.t saved ;
+  exchange         : (Endpt.id * Addr.set -> bool) option ;
+  secchan          : (Endpt.id * Time.t * Time.t * Security.key * Msecchan.status) list ref ;
+  tree             : Tree.z ref ; (* Stored state for the OPTREKEY layer *)
+  key_list         : (Key.t * Security.key) list ref 
+}
+	       
+let new_state interface = {
+  interface     = interface ;
+  exchange      = None ;
+  switch	= ref None ;
+  secchan       = ref [] ;
+  tree          = ref Tree.zempty ;
+  key_list      = ref []
+}
+			    
+let set_exchange exchange s = {
+  interface     = s.interface ;
+  exchange      = exchange ;
+  switch	= s.switch ;
+  secchan       = s.secchan ;
+  tree          = s.tree ;
+  key_list      = s.key_list 
+}
+
+let reset_state s =
+  s.switch := None
+
 (**************************************************************)
 (* Layers can specify some headers to optimize against.
  *)
@@ -56,7 +116,7 @@ type ('local,'hdr) optimize =
   | NoOpt
   | LocalNoHdr of 'local
   | FullNoHdr of 'hdr
-  | LocalSeqno of 'hdr * Event.typ * ('hdr -> Trans.seqno option) * (Trans.seqno -> 'hdr)
+  | LocalSeqno of 'hdr * Event.typ * ('hdr -> Trans.seqno option) * (Trans.seqno option -> 'hdr)
 
 (**************************************************************)
 
@@ -71,7 +131,7 @@ type ('bel,'abv) handlers_lin  = {
 }
 
 type ('bel,'abv,'state) basic =
-    View.full -> 'state * (('bel,'abv) handlers_lout -> ('bel,'abv) handlers_lin)
+  state -> View.full -> 'state * (('bel,'abv) handlers_lout -> ('bel,'abv) handlers_lin)
 
 (**************************************************************)
 (* Alias for common layers in this architecture.
@@ -87,7 +147,7 @@ let no_msg () = NoMsg
 (* Layer conversion functions.
  *)
 
-let hdr_state init hdlrs hdr_t opt args vs =
+let hdr init hdlrs hdr_t opt args vs =
   (* This check allows us to use '==' below instead of '='.
    *)
   begin 
@@ -98,65 +158,73 @@ let hdr_state init hdlrs hdr_t opt args vs =
     | FullNoHdr hdr ->
 	if Obj.is_block (Obj.repr hdr) then
 	  failwith "hdr_state:optimization attempted on block header"
-    | LocalSeqno(_,typ,_,_) ->
-        if Obj.is_block (Obj.repr typ) then
+    | LocalSeqno(hdr,_,_,_) ->
+        if Obj.is_block (Obj.repr hdr) then
 	  failwith "hdr_state:optimization attempted on block header"
     | _ -> ()
   end ;
     
   let state = init args vs in
+
   let l {up_lout=up;dn_lout=dn} =
-    let up ev abv       = up ev abv
-    and upnm ev	        = up ev NoMsg
-    and dnnm ev         = dn ev NoMsg
-    and dn =
-      (* Check if we have a header to optimize against. 
-       *)
-      match opt with
-      |	FullNoHdr nohdr ->
-	  fun ev abv hdr ->
-	    if hdr == nohdr then
-              match abv with
-	      |	Local_nohdr -> dn ev Local_nohdr
-	      |	Local_seqno(seqno) -> dn ev (Local_seqno seqno)
-	      |	_ -> dn ev (Full_nohdr abv)
-	    else		  
-	      dn ev (Full(hdr,abv))
-      | LocalSeqno(nohdr,_,det,_) ->
-	  fun ev abv hdr ->
-	    if hdr == nohdr then (
-              match abv with
-	      |	Local_nohdr -> dn ev Local_nohdr
-	      |	Local_seqno(seqno) -> dn ev (Local_seqno seqno)(*switch*)
-	      |	_ -> dn ev (Full_nohdr abv)
-	    ) else (
-	      match det hdr with
-	      |	Some seqno -> (
-		  match abv with
-		  | Local_nohdr ->
-		      dn ev (Local_seqno(seqno))
-		  | _ ->
-		      dn ev (Full(hdr,abv))
-	        )
-	      |	None ->
-		  dn ev (Full(hdr,abv))
-	    )
-      |	_ -> 
-	  fun ev abv hdr ->
-	    dn ev (Full(hdr,abv))
-    and dnlm =
-      match opt with
-      |	LocalNoHdr nohdr ->
-	  fun ev hdr ->
-	    if hdr == nohdr then
-	      dn ev Local_nohdr
-	    else dn ev (Local hdr) 
-      | _ ->
-	  fun ev hdr ->
-	    dn ev (Local hdr)
-    in
-    
     let {up_in=up;uplm_in=uplm;upnm_in=upnm;dn_in=dn;dnnm_in=dnnm} =
+      let up        = up
+      and upnm ev   = up ev NoMsg
+      and dnnm ev   = dn ev NoMsg
+      and dn =
+	(* Check if we have a header to optimize against. 
+	 *)
+	match opt with
+	| FullNoHdr nohdr ->
+	    fun ev abv hdr ->
+	      if hdr == nohdr then
+		match abv with
+		| Local_nohdr -> dn ev Local_nohdr
+		| Local_seqno _ -> 
+		    (* This is a hack to reuse the message generated above.
+		     * The types are different, but it is ok to do this here.
+		     *)
+		    dn ev (local_seqno_coerce abv)
+		| _ -> dn ev (Full_nohdr abv)
+	      else		  
+		dn ev (Full(hdr,abv))
+	| LocalSeqno(nohdr,_,det,get) ->
+	    fun ev abv hdr ->
+	      if hdr == nohdr then (
+		match abv with
+		| Local_nohdr -> dn ev Local_nohdr
+		| Local_seqno _ -> 
+		    (* Same hack as above.
+		     *)
+		    dn ev (local_seqno_coerce abv)
+		| _ -> dn ev (Full_nohdr abv)
+	      ) else (
+		match abv with
+		| Local_nohdr -> (
+		    match det hdr with
+		    | (Some _) as s -> 
+		      	dn ev (Obj.magic s : ('a,'b,'c) msg) (* Hack! *)
+		    | None ->
+		      	dn ev (Full(hdr,abv))
+		  )
+		| _ ->
+		    dn ev (Full(hdr,abv))
+	      )
+	| _ -> 
+	    fun ev abv hdr ->
+	      dn ev (Full(hdr,abv))
+      and dnlm =
+	match opt with
+	| LocalNoHdr nohdr ->
+	    fun ev hdr ->
+	      if hdr == nohdr then
+		dn ev Local_nohdr
+	      else dn ev (Local hdr) 
+	| _ ->
+	    fun ev hdr ->
+	      dn ev (Local hdr)
+      in
+    
       hdlrs state vs {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dnnm} 
     in
 
@@ -165,12 +233,12 @@ let hdr_state init hdlrs hdr_t opt args vs =
       |	FullNoHdr nohdr ->
 	  (fun ev msg -> 
 	    match msg with
-	    | Local_seqno seq -> up ev (Local_seqno seq) nohdr
-	    | Local_nohdr -> up ev Local_nohdr nohdr
+	    | Local_seqno _  -> up ev (local_seqno_coerce msg) nohdr(* same hack again*)
+	    | Local_nohdr    -> up ev Local_nohdr nohdr
 	    | Full_nohdr abv -> up ev abv nohdr
-	    | Local hdr     -> uplm ev     hdr
-	    | Full(hdr,abv) -> up   ev abv hdr
-	    | NoMsg	      -> upnm ev)
+	    | Local hdr      -> uplm ev     hdr
+	    | Full(hdr,abv)  -> up   ev abv hdr
+	    | NoMsg	     -> upnm ev)
       |	LocalNoHdr nohdr ->
 	  (fun ev msg -> 
 	    match msg with
@@ -182,12 +250,12 @@ let hdr_state init hdlrs hdr_t opt args vs =
       | LocalSeqno(nohdr,typ,_,cons) ->
 	  (fun ev msg ->
 	    match msg with
-	    | Local_seqno seq -> 
+	    | Local_seqno _ -> 
 		if Event.getType ev == typ then
-		  up ev Local_nohdr (cons seq)
+		  up ev Local_nohdr (cons (Obj.magic msg : int option)) (* Hack! *)
 		else 
-		  up ev (Local_seqno seq) nohdr
-	    | Local_nohdr -> up ev Local_nohdr nohdr
+		  up ev (local_seqno_coerce msg) nohdr (* same hack yet again *)
+	    | Local_nohdr   -> up ev Local_nohdr nohdr
 	    | Full_nohdr abv -> up ev abv nohdr
 	    | Local hdr     -> uplm ev     hdr
 	    | Full(hdr,abv) -> up   ev abv hdr
@@ -212,16 +280,12 @@ let hdr_state init hdlrs hdr_t opt args vs =
   in (state,l)
 
 (**************************************************************)
-
-let hdr init hdlr hdr_t opt args vs =
-  let (_,l) = hdr_state init hdlr hdr_t opt args vs in
-  ((),l)
-
-(**************************************************************)
-(* Layer conversion function with no optimizations.
+(* Layer conversion function with no optimizations.  This is
+ * the same as the above, but without the header
+ * optimizations.  
  *)
 
-let hdr_state_noopt init hdlrs args vs =
+let hdr_noopt init hdlrs args vs =
   let state = init args vs in
   let l {up_lout=up;dn_lout=dn} =
     let up ev abv       = up ev abv
@@ -239,7 +303,7 @@ let hdr_state_noopt init hdlrs args vs =
     | Local hdr     -> uplm ev     hdr
     | Full(hdr,abv) -> up   ev abv hdr
     | NoMsg	    -> upnm ev
-    | _ -> failwith "hdr_state_noopt:sanity"
+    | _ -> failwith "hdr_noopt:sanity"
     and dn ev abv = match abv with
     | NoMsg         -> dnnm ev
     | _             -> dn ev abv
@@ -249,92 +313,21 @@ let hdr_state_noopt init hdlrs args vs =
   in (state,l)
 
 (**************************************************************)
-(* Record containing overall fields for protocol stacks.
- * Assumes no layers are repeated.  
+(* These are strings used in lots of layers, so we share
+ * them all here.
  *)
-
-type state = {
-  interface     : Appl_intf.t ;
-  switch	: Time.t saved
-}
-
-let new_state interface = {
-  interface     = interface ;
-  switch	= ref None
-}
-
-let reset_state s =
-  s.switch := None
-
-(**************************************************************)
-
-type lyr = (unit,unit,unit) basic
-
-let table = 
-  let table = Hashtbl.create 10 in
-  Trace.install_root (fun () ->
-    [sprintf "LAYER:#layers=%d" (hashtbl_size table)]
-  ) ;
-  table
-
-let install name layer = 
-  let name = String.uppercase name in
-  Hashtbl.add table name layer
-
-let lookup name = 
-  let name = String.uppercase name in
-  Hashtbl.find table name
-
-(**************************************************************)
-
-let magic o = Obj.magic (Obj.repr o)
-
-type ('args,'b,'c,'d) init = 
-  ('args -> ('b  ,'c  ,'d  ) basic) -> 
-  state -> (unit,unit,unit) basic
-
-let coerce (l : ('a,'b,'c)basic) = (magic l : lyr)
-
-let init l s         = coerce (l ())
-let initSwitch l s   = coerce (l s.switch)
-let initTop_appl l s = coerce (l (s.interface))
-let initNamed l s    = coerce (l "???")
-
-(**************************************************************)
-
-let procurers = ref ["internal_table",lookup]
-
-let finder name lookup =
-  procurers := !procurers @ [name,lookup]
-
-let procure_hide name state =
-  let rec loop = function
-  | [] ->
-      eprintf "LAYER:couldn't find layer:%s\n" name ;
-      failwith "no such layer"
-  | (_,lookup) ::tl ->
-      try
-	let l = lookup name in
-	let l = (l : state -> lyr) in
-	let l = magic l in
-	let l = (l : state -> ('a,'b,'c) basic) in
-	l state
-      with Not_found -> (
-	loop tl
-      )
-  in loop !procurers
-
-let look_again name =
-  let l = (lookup name : state -> lyr) in
-  let l = (magic l : state -> ('a,'b,'c) basic) in
-  l
-
-(**************************************************************)
-
-let config_print () =
-  eprintf "  Procure:Installed Layers\n" ;
-  Hashtbl.iter (fun name _ ->
-    eprintf "	%s\n" name
-  ) table
+let syncing = "SYNCING"
+let block = "BLOCK"
+let buffer = "BUFFER"
+let unknown_local = "unknown local message"
+let bad_header = "bad header"
+let bad_up_event = "bad up event"
+let layer_fail dump vf s debug =
+  let failwith m = dump vf s ; Trace.make_failwith debug m in
+  failwith
+let layer_dump name f vf s =
+  eprintf "%s:dump:%s\n" name (fst vf).View.name ;
+  Array.iter (fun s -> eprintf "  %s" s) (f vf s)
+let layer_dump_simp name = layer_dump name (fun _ _ -> [||])
 
 (**************************************************************)
