@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -14,6 +14,8 @@
 open Trans
 (**************************************************************)
 
+(* The type of parameters.
+ *)
 type t =
   | String of string
   | Int of int
@@ -21,13 +23,28 @@ type t =
   | Time of Time.t
   | Float of float
 
+(* Parameter lists are (name,value) association lists.
+ *)
 type tl = (name * t) list
 
+(* Add a parameter to the defaults.
+ *)
 val default : name -> t -> unit
-val print_defaults : unit -> unit
+
+(* Lookup a parameter in a param list.
+ *)
 val lookup : tl -> name -> t
+
+(* Lookup a particular type of parameter.
+ *)
 val string : tl -> name -> string
 val int : tl -> name -> int
 val bool : tl -> name -> bool
 val time : tl -> name -> Time.t
 val float : tl -> name -> float
+
+val to_string : (name * t) -> string
+
+(* Print out default settings.
+ *)
+val print_defaults : unit -> unit
