@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -30,13 +30,3 @@ val string_of_l : l -> name
 val l_of_string : name -> l
 
 (**************************************************************)
-
-val raw : id
-
-(**************************************************************)
-(* Temporary.
- *)
-
-val transis_debug : id
-
-(**************************************************************)
