@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -12,29 +12,29 @@
 (* Authors: Mark Hayden, 6/95 *)
 (**************************************************************)
 open Trans
+open Buf
+(**************************************************************)
 
 (* Type of keys in Ensemble.
  *)
 type key = 
   | NoKey
-  | Common of string
+  | Common of Buf.t
 
 (* Create a new random key.  This is not cryptographically
- * secure.
+ * secure.  
  *)
+(*
 val create : unit -> key
+*)
 
 (* Get the string contents of a key.
  *)
 val str_of_key : key -> string
+val buf_of_key : key -> Buf.t
 
-(* Get a textual representation of the key
- * for printing out to the user.
+(* Get a textual representation of the key for printing out
+ * to the user.  
  *)
 val string_of_key_short : key -> string
 val string_of_key : key -> string
-
-(* Sign a portion of a buffer with a key.
- *)
-val sign : key -> buf -> ofs -> len -> string
-
