@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -11,8 +11,8 @@
 (* STACK_ID.ML *)
 (* Author: Mark Hayden, 3/96 *)
 (**************************************************************)
-let name = Trace.source_file "STACK_ID"
-let failwith s = failwith (Util.failmsg name s)
+let name = Trace.file "STACK_ID"
+let failwith = Trace.make_failwith name
 (**************************************************************)
 
 type t =
@@ -21,11 +21,12 @@ type t =
   | Gossip
   | Unreliable
 
-let mapping = [
+let mapping = [|
   "Primary", Primary ;
   "Gossip", Gossip ;
   "Unreliable", Unreliable ;
   "Bypass", Bypass
-] 
+|] 
 
-let id_of_string,string_of_id = Util.make_map name "id" mapping
+let id_of_string s = Util.id_of_string name mapping s
+let string_of_id i = Util.string_of_id name mapping i
