@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -11,32 +11,31 @@
 (* UNIQUE.MLI *)
 (* Author: Mark Hayden, 8/96 *)
 (**************************************************************)
+open Trans
+
+(* Id generator.
+ *)
+type t
 
 (* Unique identifiers.
  *)
 type id
 
-(* Constructor.
+(* Constructors.
  *)
-val id : unit -> id
+val create : inet -> incarn -> t
+val id : t -> id
 
 (* Display functions.
  *)
 val string_of_id : id -> string
 val string_of_id_short : id -> string
-val string_of_id_realshort : id -> string
+val string_of_id_very_short : id -> string
 
 (* Hash an id.
  *)
 val hash_of_id : id -> int
 
-(* Are these from the same process?
- *)
-val same_process : id -> id -> bool
-
 (**************************************************************)
-(* For transports.
- *)
-val mux : id -> int			(* not for casual use *)
-val install_port : int -> unit
+val set_port : t -> port -> unit
 (**************************************************************)
