@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -14,11 +14,21 @@
 (* More array operations.  All are in-place (the first arg
  * is updated).
  *)
-
 val maxify : int array -> int array -> unit
 val minify : int array -> int array -> unit
 val mergify : bool array -> bool array -> unit
-val fillify : 'a array -> 'a -> unit
 val setify : 'a array -> 'a array -> unit
+val fillify : 'a array -> 'a -> unit
+
+val fsetify : 'a array -> 'a Arrayf.t -> unit
+val fmaxify : int array -> int Arrayf.t -> unit
+val fminify : int array -> int Arrayf.t -> unit
+val fmergify : bool array -> bool Arrayf.t -> unit
+
+(* These are purely functional versions.
+ *)
+val ffmaxify : int Arrayf.t -> int Arrayf.t -> int Arrayf.t
+val ffminify : int Arrayf.t -> int Arrayf.t -> int Arrayf.t
+val ffmergify : bool Arrayf.t -> bool Arrayf.t -> bool Arrayf.t
 
 (**************************************************************)
