@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -27,7 +27,7 @@ let create name = {
 let set t =
   if t.data then (
     eprintf "ONCE:variable set 2nd time:%s\n" t.name ;
-    failwith "sanity" ;
+    failwith sanity ;
   ) ;
   t.data <- true
   
