@@ -1,34 +1,34 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
  *)
 (**************************************************************)
 (**************************************************************)
-(* Resource.MLI *)
+(* RESOURCE.MLI *)
 (* Author: Mark Hayden, 4/96 *)
 (**************************************************************)
+open Trans
+(**************************************************************)
 
 type ('a,'b) t
 
-type name = string
-
-val create 	: 
+val create : 
   name -> 
   ('a -> 'b -> unit) ->			(* add *)
-  ('a -> unit) -> 			(* remove *)
+  ('a -> 'b -> unit) ->			(* remove *)
   (('a,'b) t -> unit) ->		(* change *)
+  (('a,'b) t -> unit) ->		(* minimal change *)
   ('a,'b) t
+
 val simple 	: name -> ('a,'b) t
-val add       	: ('a,'b) t -> 'a -> 'b -> unit
+val add       	: ('a,'b) t -> debug -> 'a -> 'b -> unit
 val remove 	: ('a,'b) t -> 'a -> unit
-val to_list	: ('a,'b) t -> 'a list
-val to_list2	: ('a,'b) t -> 'b list
-val to_list_ref	: ('a,'b) t -> 'a list ref
-val to_list2_ref : ('a,'b) t -> 'b list ref
+
+val to_array	: ('a,'b) t -> 'b Arrayf.t
 
 val info	: ('a,'b) t -> string
-val verbose 	: bool ref
+val to_string   : ('a,'b) t -> string
