@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.40)
- *  Copyright 1997 Cornell University
+ *  Ensemble, (Version 0.70p1)
+ *  Copyright 2000 Cornell University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -20,15 +20,23 @@ type rank	= int
 type nmembers   = int
 type data_int	= int
 type seqno	= int
-type buf	= string
-type ofs	= int
-type len	= int
 type ltime      = int
 type bitfield	= int
 type fanout     = int
 type name       = string
 type debug      = string
 type inet       = Hsys.inet
+type eth        = Hsys.eth
 type origin     = rank
+type primary    = bool
+type pid        = int
 
-let md5len = String.length (Digest.string "") 
+module type OrderedType =
+  sig
+    type t
+    val zero : t
+    val ge : t -> t -> bool
+  end
+
+
+(**************************************************************)
